#!/bin/bash

################################################################################
#
# Citrix Virtual Apps & Desktops For Linux Script: Machine Creation Service
# Copyright (c) Citrix Systems, Inc. All Rights Reserved.
#

source /var/xdl/mcs/mcs_util.sh

fname=$(basename $0)
scriptName=${fname%.*}  # Script name without extension
logFile="/var/log/$scriptName.log"


function conf_hostname()
{
    log "Debug: Enter conf_hostname"
    
    hosts_file="/etc/hosts"
    hostname_file="/etc/hostname"
    content="127.0.0.1 ${NEW_HOSTNAME}.${realm} $NEW_HOSTNAME localhost localhost.localdomain localhost4 localhost4.localdomain4"

    echo "$content" > "$hosts_file"
    echo "$NEW_HOSTNAME" > "$hostname_file"
    hostname "$NEW_HOSTNAME"

    sysctl -w kernel.hostname="$NEW_HOSTNAME"  2>&1 >> "$logFile"
    log "Debug: Exit conf_hostname"
}

function conf_ntp()
{
    log "Debug: Enter conf_ntp"
    
    ntp_file="/etc/chrony.conf"
    sed -i -e '/^server.*iburst$/ d' "$ntp_file"
    sed -i -e '/^pool.*iburst$/ d' "$ntp_file"
    echo "server $NTP_SERVER iburst" >> "$ntp_file"

    # sync time with ntp server
    log_echo "stop chronyd daemon"
    systemctl stop chronyd.service 2>&1 >> "$logFile"
    log_echo "start chronyd daemon"
    systemctl start chronyd.service 2>&1 >> "$logFile"

    log "Debug: Exit conf_ntp"
}

function conf_winbind_krb5()
{
    log "Debug: Enter function conf_winbind_krb5"
    krbFile="/etc/krb5.conf"

    echo "[libdefaults]
    default_tgs_enctypes = rc4-hmac des-cbc-crc des-cbc-md5
    default_tkt_enctypes = rc4-hmac des-cbc-crc des-cbc-md5
    default_ccache_name = FILE:/tmp/krb5cc_%{uid}
    default_realm = $REALM
    dns_lookup_realm = true
    dns_lookup_kdc = true

[realms]
$REALM = {
    kdc = $AD_FQDN
}

[domain_realm]
    ${realm} = $REALM
    .${realm} = $REALM">"$krbFile"

    log "Debug: Exit function conf_winbind_krb5"
}

function conf_sssd_krb5()
{
    log "Debug: Enter function conf_sssd_krb5"
    krbFile="/etc/krb5.conf"

    echo "[logging]
    default = FILE:/var/log/krb5libs.log
    kdc = FILE:/var/log/krb5kdc.log
    admin_server = FILE:/var/log/kadmind.log

[libdefaults]
    default_ccache_name = FILE:/tmp/krb5cc_%{uid}
    default_realm = $REALM
    dns_lookup_realm = true
    dns_lookup_kdc = true
    ticket_lifetime = 24h
    renew_lifetime = 7d
    forwardable = true
    rdns = false

[realms]
$REALM = {
    kdc = $AD_FQDN
}

[domain_realm]
    ${realm} = $REALM
    .${realm} = $REALM">"$krbFile"

    log "Debug: Exit function conf_sssd_krb5"
}

function conf_winbind()
{
    log "Debug: Enter conf_winbind"

    # stop winbind daemon
    log "Stopping winbind service"
    systemctl stop winbind 2>&1 >> "$logFile"

    # configure winbind authentication
    args="--disablecache --disablesssd --disablesssdauth --enablewinbind --enablewinbindauth --disablewinbindoffline --smbsecurity=ads --smbworkgroup=$WORKGROUP --smbrealm=$REALM --krb5realm=$REALM --krb5kdc=$AD_FQDN --winbindtemplateshell=/bin/bash --enablemkhomedir --updateall --enablekrb5kdcdns --enablekrb5realmdns"

    log "Execute command: authconfig $args"
    authconfig $args 2>&1 | tee -a "$logFile"
    [[ "$?" -ne "0" ]] && log_echo "failed to execute command: authconfig $args [Error]"

    smbFile="/etc/samba/smb.conf"
    pamFile="/etc/security/pam_winbind.conf"

    # Customize /etc/samba/smb.conf
    `sed -i '/kerberos method =.*$/d' "$smbFile"`
    `sed -i '/winbind refresh tickets =.*$/d' "$smbFile"`      # del line in case user execute the script multi times
    `sed -i '/\[global\]/a winbind refresh tickets = true' "$smbFile"`
    `sed -i '/\[global\]/a kerberos method = secrets and keytab' "$smbFile"`

    conf_winbind_krb5             #Customize /etc/krb5.conf in case user execute the script multi times

    # Customize /etc/security/pam_winbind.conf
    if [[ -f "$pamFile" ]]; then
        `sed -i 's/.*krb5_auth =.*$/krb5_auth = yes/g' "$pamFile"`
        `sed -i 's/.*krb5_ccache_type =.*$/krb5_ccache_type = FILE/g' "$pamFile"`
        `sed -i 's/.*mkhomedir =.*$/mkhomedir = yes/g' "$pamFile"`
    else
    echo "[global]
    krb5_auth = yes
    krb5_ccache_type = FILE
    mkhomedir = yes">"$pamFile" 
    fi

    log "Debug: Exit conf_winbind"
}


function conf_sssd()
{
    log "Debug: Enter conf_sssd"

    # configure winbind authentication
    args="--smbsecurity=ads --smbworkgroup=$WORKGROUP --smbrealm=$REALM --krb5realm=$REALM --krb5kdc=$AD_FQDN --update --enablekrb5kdcdns --enablekrb5realmdns"

    log "Execute command: authconfig $args"
    authconfig $args 2>&1 | tee -a "$logFile"
    [[ "$?" -ne "0" ]] && log_echo "failed to execute command: authconfig $args [Error]"

    smbFile="/etc/samba/smb.conf"

    # Customize /etc/samba/smb.conf
    `sed -i '/kerberos method =.*$/d' "$smbFile"`
    `sed -i '/\[global\]/a kerberos method = secrets and keytab' "$smbFile"`

    conf_sssd_krb5             #Customize /etc/krb5.conf in case user execute the script multi times

    # Customize /etc/sssd/sssd.conf
    conf_sssd_sssd

    log "Debug: Exit conf_sssd"
}

function conf_sssd_sssd()
{
    log "Debug: Enter function conf_sssd_sssd"
    sssdFile="/etc/sssd/sssd.conf"
    echo "[sssd]
    config_file_version = 2
    domains = $realm
    services = nss,pam

[domain/$realm]
    id_provider = ad
    auth_provider = ad
    access_provider = ad
    ldap_id_mapping = true
    ldap_schema = ad

    ad_server = $AD_FQDN
    ad_domain = $realm

    krb5_ccachedir = /tmp
    krb5_ccname_template = FILE:%d/krb5cc_%U
    default_shell = /bin/bash
    fallback_homedir = /home/%d/%u">"$sssdFile"
    
    # Set the file ownership and permissions on sssd.conf
    chown root:root "$sssdFile"
    chmod 0600 "$sssdFile"
    restorecon  "$sssdFile"

    #enable sssd service
    args="--enablesssdauth --enablesssd --enablemkhomedir --update"

    log "Execute command: authconfig $args"
    authconfig $args 2>&1 | tee -a "$logFile"
    [[ "$?" -ne "0" ]] && log_echo "failed to execute command: authconfig $args [Error]"

    #start sssd service to create secret file
#    systemctl start sssd  2>&1 >> "$logFile"
#    sleep 2    

    log "Debug: Exit conf_sssd_sssd"
}


## dbus needs to know the auth method.
function conf_winbind_nsswitch()
{
    log "Debug: Enter conf_winbind_nsswitch"

    # Customize /etc/nsswitch.conf
    nsswitchFile="/etc/nsswitch.conf"
    passwdLine="$(cat $nsswitchFile |grep passwd |grep -v sss | grep winbind 2>&1)"
    shadowLine="$(cat $nsswitchFile |grep shadow |grep -v sss | grep winbind 2>&1)"
    groupLine="$(cat $nsswitchFile |grep group | grep -v netgroup | grep -v sss | grep winbind 2>&1)"
    netgroupLine="$(cat $nsswitchFile |grep netgroup |grep sss 2>&1)"
    serviceLine="$(cat $nsswitchFile |grep services |grep sss 2>&1)"
    automountLine="$(cat $nsswitchFile |grep automount |grep sss 2>&1)"
    [[ -z "$passwdLine" ]] && `sed -i 's/^passwd:.*$/passwd:     files winbind/g' "$nsswitchFile"`
    [[ -z "$shadowLine" ]] && `sed -i 's/^shadow:.*$/shadow:     files winbind/g' "$nsswitchFile"`
    [[ -z "$groupLine" ]] && `sed -i 's/^group:.*$/group:      files winbind/g' "$nsswitchFile"`
    [[ ! -z "$serviceLine" ]] && `sed -i 's/^services:.*$/services:   files/g' "$nsswitchFile"`
    [[ ! -z "$netgroupLine" ]] && `sed -i 's/^netgroup:.*$/netgroup:   files/g' "$nsswitchFile"`
    [[ ! -z "$automountLine" ]] && `sed -i 's/^automount:.*$/automount:  files/g' "$nsswitchFile"`

    log "Debug: Exit conf_winbindnsswitch"
}

function conf_sssd_nsswitch()
{
    log "Debug: Enter conf_sssd_nsswitch"

    # Customize /etc/nsswitch.conf
    nsswitchFile="/etc/nsswitch.conf"
    passwdLine="$(cat $nsswitchFile |grep passwd |grep sss 2>&1)"
    shadowLine="$(cat $nsswitchFile |grep shadow |grep sss 2>&1)"
    groupLine="$(cat $nsswitchFile |grep group | grep -v netgroup | grep sss 2>&1)"
    netgroupLine="$(cat $nsswitchFile |grep netgroup |grep sss 2>&1)"
    serviceLine="$(cat $nsswitchFile |grep services |grep sss 2>&1)"
    automountLine="$(cat $nsswitchFile |grep automount |grep sss 2>&1)"
    [[ -z "$passwdLine" ]] && `sed -i 's/^passwd:.*$/passwd:     files sss/g' "$nsswitchFile"`
    [[ -z "$shadowLine" ]] && `sed -i 's/^shadow:.*$/shadow:     files sss/g' "$nsswitchFile"`
    [[ -z "$groupLine" ]] && `sed -i 's/^group:.*$/group:      files sss/g' "$nsswitchFile"`
    [[ -z "$serviceLine" ]] && `sed -i 's/^services:.*$/services:   files sss/g' "$nsswitchFile"`
    [[ -z "$netgroupLine" ]] && `sed -i 's/^netgroup:.*$/netgroup:   files sss/g' "$nsswitchFile"`
    [[ -z "$automountLine" ]] && `sed -i 's/^automount:.*$/automount:  files sss/g' "$nsswitchFile"`

    log "Debug: Exit conf_sssd_nsswitch"
}


function restart_sssd_service()
{
    log "Debug: Enter restart_sssd_service"

    log_echo "restarting sssd service"
    systemctl restart sssd 2>&1 >> "$logFile"

    log "Debug: Exit restart_sssd_service"

}

function restart_winbind_service()
{
    log "Debug: Enter restart_winbind_service"

    # start winbind service
    log_echo "starting winbind service ..."
    systemctl restart winbind 2>&1 >> "$logFile"

    log "Debug: Exit restart_winbind_service"
}

# this is called by ad_join service
function join_domain_setup_vda()
{
    log "Debug: Enter join_domain_setup_vda"
    conf_ntp
    if [ "$AD_INTEGRATION" == "winbind" ]; then
        conf_winbind
        join_domain
        restart_winbind_service
    else
        conf_sssd
        join_domain
        restart_sssd_service
    fi
    setup_vda
    log "Debug: Exit join_domain_setup_vda"
}

# this is called before dbus service start
function config_file()
{
    log "Debug: Enter config_file"
    conf_hostname
    if [ "$AD_INTEGRATION" == "winbind" ]; then
        conf_winbind_nsswitch
    else
        conf_sssd_nsswitch
    fi
    log "Debug: Exit config_file"
}

check_user
check_ntfs_3g_rpm
read_id_disk

if [ "$1" == "--config" ] ; then
    config_file
    exit 0
elif [ "$1" == "--setup" ] ; then
    join_domain_setup_vda
    exit 0
else
    exit 1
fi
