#!/bin/bash

###############################################################################
#
# Citrix Virtual Apps & Desktops For Linux Script: Machine Creation Service
# Copyright (c) Citrix Systems, Inc. All Rights Reserved.
#

source /var/xdl/mcs/mcs.conf

build_dir="/opt/Citrix/VDA/lib64/mcs"
mcs_sysd_unit_file="ad_join.service"
mcs_sysd_unit_file_rhel6="ad_join"
mcs_boot_script="ad_join.sh"
mcs_script_folder="/var/xdl/mcs"

id_disk_mnt_point="/mnt/iddisk"
ad_info_file_path="/PvsVm/CTXSOSID.INI"
SAMBA_SECRETS="/var/lib/samba/private/secrets.tdb"
KEYTAB="/etc/krb5.keytab"

AD_INTEGRATION=`echo $AD_INTEGRATION | tr 'A-Z' 'a-z'`
NEW_HOSTNAME=""
new_hostname=""
AD_FQDN=""
DDCS=""
domain=""
realm=""
PASSWORD=""
DNSDISID=""
DDC=""
REALM=""
retry_max=10

function log()
{
    PREFIX=$(date +"%b %d %T")
    echo "${PREFIX} ${1}" >> $logFile
}

function log_echo()
{
    log "$1"
    echo "$1"
}

function create_log_file()
{
    if [[ ! -f "$logFile" ]]; then
        touch "$logFile"
        if [[ "$?" -ne "0" ]]; then
          log_echo "Create log file $logFile failed" 
        fi
    fi

    echo "#### Begin $scriptName ####">>$logFile
    str="`date "+%Y-%m-%d %H:%M:%S"`"
    echo $str>>$logFile
}

#
# Check user, only root user has the permission to run this script
#
function check_user()
{
    if [ "$(id -u)" != 0 ]; then
        log_echo "Error: The script must be run by root user"
        exit 1
    fi
}

function check_string_empty()
{
    if [ -z "$2" ]; then
    log_echo "Error: $1 is empty, existing ..."
    exit 1
    fi
}


function check_ntfs_3g_rpm()
{
    log "Debug: Enter check_ntfs_3g_rpm"

    # check whether ntfs-3g is installed
    FOUND_NTFS_3G=$(rpm -qa | grep -i "ntfs-3g" > /dev/null 2>&1; echo $?)
    if [ ! $FOUND_NTFS_3G ]; then
        log_echo "Error: ntfs-3g not installed, exit"
        exit 1
    else
        log_echo "Info: ntfs-3g is installed."
    fi

    log "Debug: Exit check_ntfs_3g_rpm"
}


function check_ntfs_3g_deb()
{
    log "Debug: Enter check_ntfs_3g_deb"

    # check whether ntfs-3g is installed
    FOUND_NTFS_3G=$(dpkg -l | grep -i "ntfs-3g" > /dev/null 2>&1; echo $?)
    if [ ! $FOUND_NTFS_3G ]; then
        log_echo "Error: ntfs-3g not installed, exit"
        exit 1
    else
        log_echo "Info: ntfs-3g is installed."
    fi

    log "Debug: Exit check_ntfs_3g_deb"
}

function install_mcs_scripts_rhel6()
{
    log "Debug: Enter install_mcs_scripts"

    pushd $build_dir

    log_echo "installing mcs systemd unit file: ${mcs_sysd_unit_file_rhel6} ... "
    # service file needs exec permission on rhel6
    install -o root -m 0755 ${mcs_sysd_unit_file_rhel6} /etc/init.d/

    log "setting ${mcs_sysd_unit_file} to start on boot"
    /sbin/chkconfig ${mcs_sysd_unit_file_rhel6} on

    log_echo "installing MCS boot script ${mcs_boot_script}"
    install -o root -m 0755 ${mcs_boot_script} ${mcs_script_folder}

    popd

    log "Debug: Exit install_mcs_scripts"
}


## install mcs related scripts and services
function install_mcs_scripts()
{
    log "Debug: Enter install_mcs_scripts"

    pushd $build_dir

    log_echo "installing mcs systemd unit file: ${mcs_sysd_unit_file} ... "
    install -o root -m 0644 ${mcs_sysd_unit_file} /etc/systemd/system/

    log "reloading systemd daemon..."
    systemctl daemon-reload

    log "setting ${mcs_sysd_unit_file} to start on boot"
    systemctl enable ${mcs_sysd_unit_file}

    log "installing MCS boot script ${mcs_boot_script}"
    install -o root -m 0755 ${mcs_boot_script} ${mcs_script_folder}

    popd

    log "Debug: Exit install_mcs_scripts"
}

function hack_dbus_service_file()
{
    log "Debug: Enter hack_dbus_service_file"

    service_path=$1
    log "dbus service file path is: $service_path"

    sed -i -e '/^ExecStartPre=.*$/ d' "$service_path"
    sed -i "/\[Service\]/a ExecStartPre=${mcs_script_folder}/${mcs_boot_script} --config" "$service_path"
    log "reloading systemd daemon..."
    systemctl daemon-reload

    log "Debug: Exit hack_dbus_service_file"
}

function read_id_disk()
{
    log "Debug: Enter read_id_disk"

    # test whether id disk is attached
    iddisk_dev_name=$(lsblk | grep 13M | sed 's/^[^[:alnum:]]*\([[:alnum:]]\+\).*$/\1/')
    if [ -z "$iddisk_dev_name" ]; then
        log_echo "Error: No id disk is attached to the machine. Exit!"
        exit 0
    else
        log_echo "Info: identity disk is attach to /dev/$iddisk_dev_name"
    fi

    # create the mount point folder if it does not exist
    [ ! -d ${id_disk_mnt_point} ] && mkdir -p ${id_disk_mnt_point}

    # check whether the id disk is already mounted, if so, umount it
    if mount | grep "/dev/$iddisk_dev_name" ; then
        echo "/dev/$iddisk_dev_name already mounted, umount it"
        umount /dev/$iddisk_dev_name
    fi

    # mount the id disk to ${id_disk_mnt_point}
    echo "mounting /dev/$iddisk_dev_name to ${iddisk_dev_name}"
    mount -t ntfs-3g -o umask=077 /dev/$iddisk_dev_name ${id_disk_mnt_point}
    

    # check whether the id disk is an image preparation disk or id disk
    img_prep_signature=`cat ${id_disk_mnt_point}/PvsVm/CTXSOSID.INI | grep '\[ImagePreparation\]' | sed 's/\s//g'`
    if [ "$img_prep_signature" == "[ImagePreparation]" ]; then
        # extract the STATUS file name
        status_filename=`cat ${id_disk_mnt_point}/PvsVm/CTXSOSID.INI | grep 'StatusFile' | cut -d'=' -f2 | sed 's/\s//g'`

        # write status file to the root of ${id_disk_mnt_point}
        cat <<-EOF > ${id_disk_mnt_point}/${status_filename}
<?xml version="1.0"?>
<PreparationResults>
  <PreparationResults>
    <Status>Complete</Status>
    <CycleCount>1</CycleCount>
    <StepStatus>
      <OfficeRearm>Success</OfficeRearm>
      <EnableDHCP>Success</EnableDHCP>
      <PrepareMSMQ>Success</PrepareMSMQ>
      <OsRearm>Success</OsRearm>
      <PrepJoinDomain>Success</PrepJoinDomain>
    </StepStatus>
    <ThirdPartyStatus />
    <Log>
</Log>
  </PreparationResults>
</PreparationResults>
EOF
        # flush the image preparation status file to disk
        log "flushing the status file to id disk..."
        sync ${id_disk_mnt_point}/${status_filename}
        #shutdown image prep machine
        sleep 5
        log_echo "shutting down the machine..."
        shutdown -h now

        log "exiting ..."
        exit 0
    fi


    # extract information from id disk
    NEW_HOSTNAME=`cat ${id_disk_mnt_point}${ad_info_file_path} | grep 'LongHostName' | cut -d'=' -f2 | sed 's/\s//g'`
    AD_FQDN=`cat ${id_disk_mnt_point}${ad_info_file_path} | grep 'DCName' | cut -d'=' -f2 | sed 's/\\\\//g' | sed 's/\s//g'`
    DDCS=`cat ${id_disk_mnt_point}${ad_info_file_path} | grep 'ListOfDDCs' | cut -d'=' -f2 | sed 's/\s//g'`
    realm=`cat ${id_disk_mnt_point}${ad_info_file_path} | grep 'DomainName' | cut -d'=' -f2 | sed 's/\s//g'`

    # password is base64 encoded, so it need to be decoded here
    PASSWORD=`cat ${id_disk_mnt_point}${ad_info_file_path} | grep 'MachinePassword' | cut -d'=' -f2 | sed 's/\s//g' | base64 -d`

    # DOMAIN SID
    DNSDISID=`cat ${id_disk_mnt_point}${ad_info_file_path} | grep 'DNSDISid' | cut -d'=' -f2 | sed 's/\s//g'`

    check_string_empty "NEW_HOSTNAME" $NEW_HOSTNAME
    check_string_empty "AD_FQDN" $AD_FQDN
    check_string_empty "DDCS" $DDCS
    check_string_empty "realm" $realm
    check_string_empty "PASSWORD" $PASSWORD
    check_string_empty "DNSDISID" $DNSDISID

    log_echo "NEW_HOSTNAME = $NEW_HOSTNAME"
    log_echo "AD_FQDN = $AD_FQDN"
    log_echo "DDCS = $DDCS"
    log_echo "realm = $realm"
    log_echo "PASSWORD = $PASSWORD"
    log_echo "DNSDISID = $DNSDISID"


    DDCS=`echo ${DDCS} | tr ',' ' '`
    REALM=`echo $realm | tr 'a-z' 'A-Z'`
    realm_lower=`echo $realm | tr 'A-Z' 'a-z'`
    new_hostname=`echo $NEW_HOSTNAME | tr 'A-Z' 'a-z'`
    domain=`echo ${realm} | cut -d'.' -f1`
    DOMAIN=`echo ${domain} | tr 'a-z' 'A-Z'`

    # Get WORKGROUP info
    if [[ -z "$WORKGROUP" ]]; then
        log_echo "WORKGROUP is not set in config file, set to $DOMAIN"
        WORKGROUP=$DOMAIN
    else
        log_echo "Read WORKGROUP from config file: $WORKGROUP"
    fi
    # Get NTP_SERVER info
    if [[ -z "$NTP_SERVER" ]]; then
        log_echo "NTP_SERVER is not set in config file, set to $AD_FQDN"
        NTP_SERVER=$AD_FQDN
    else
        log_echo "Read NTP_SERVER from config file: $NTP_SERVER"
    fi
    # Get LDAP_LIST info
    if [[ -z "$LDAP_LIST" ]]; then
        LDAP_LIST="${AD_FQDN}:389"
        log_echo "LDAP_LIST is not set in config file, set to $LDAP_LIST"
    else
        log_echo "Read LDAP_LIST from config file: $LDAP_LIST"
    fi

    log_echo "domain = $domain"

    log "Debug: Exit read_id_disk"
}

function conf_resolv_conf()
{
    log "Debug: Enter conf_resolv_conf"

    dnsFile="/etc/resolv.conf"

    `sed -i '/^nameserver.*$/d' "$dnsFile"`   
    [[ -n "$dns1" ]] && str1="nameserver $dns1" && echo "$str1">"$dnsFile"
    [[ -n "$dns2" ]] && str1="nameserver $dns2" && echo "$str1">>"$dnsFile"
    [[ -n "$dns3" ]] && str1="nameserver $dns3" && echo "$str1">>"$dnsFile"
    [[ -n "$dns4" ]] && str1="nameserver $dns4" && echo "$str1">>"$dnsFile"

    log "Debug: exit conf_resolv_conf"
}

function join_domain()
{
    log "Debug: Enter join_domain"

    # update domain sid
    log "Updating SECRET/SID/${DOMAIN} to ${DNSDISID}"
    net setdomainsid ${DNSDISID} 2>&1 >> "$logFile"


    log "Changing SECRETS/SALTING_PRINCIPAL/DES/${REALM} to host/${NEW_HOSTNAME}.${domain}@${DOMAIN}"
    tdbtool ${SAMBA_SECRETS} store SECRETS/SALTING_PRINCIPAL/DES/${REALM} "host/${new_hostname}.${realm_lower}@${REALM}\0" 2>&1 >> "$logFile"


    # update machine password
    log "Changing SECRETS/MACHINE_PASSWORD/${WORKGROUP} to '${PASSWORD}'"
    tdbtool ${SAMBA_SECRETS} store SECRETS/MACHINE_PASSWORD/${WORKGROUP} "${PASSWORD}\0" 2>&1 >> "$logFile"

    # write second channel type
    log "Changing SECRETS/MACHINE_SEC_CHANNEL_TYPE/${WORKGROUP} to \02\00\00\00"
    tdbtool ${SAMBA_SECRETS} store "SECRETS/MACHINE_SEC_CHANNEL_TYPE/${WORKGROUP}" "\02\00\00\00"

    net cache flush

    # update keytab file
    log_echo "Updating $KEYTAB" 
    if [ -f $KEYTAB ]; then
        rm -f $KEYTAB
    fi

    for ((retry_num=1; retry_num<=$retry_max; retry_num++))
    do
        log_echo "Retry time: $retry_num, sleeping 2 seconds before creating keytab file..."
        sleep 2
        net ads keytab create -P 2>&1 >> "$logFile"
        if [[ "$?" -ne "0" ]]; then
            log_echo "Error: Kerboros keytab creation failed!"
        else
            log_echo "Kerboros keytab creation succeed!"
            break
        fi
    done

    sleep 1
    # update dns record
    log "updating DNS record ..."
    net ads dns register -P

    sleep 1

    STATUS=$(net ads testjoin > /dev/null; echo $?)
    if [ $STATUS -eq 0 ]; then
        log_echo "Machine join passed testing"
    else
        log_echo "Error: Machine join failed testing"
    fi

    log "Debug: Exit join_domain"
}


function setup_vda()
{
    log "Debug: Enter setup_vda"
    log_echo "updating VDA configurations ..."
    if [ "$AD_INTEGRATION" == "winbind" ]; then
        CTX_AD_INTEGRATION=1
    else
        CTX_AD_INTEGRATION=4
    fi
    export CTX_XDL_SUPPORT_DDC_AS_CNAME=${SUPPORT_DDC_AS_CNAME}
    export CTX_XDL_DDC_LIST=${DDCS}
    export CTX_XDL_VDA_PORT=${VDA_PORT}
    export CTX_XDL_REGISTER_SERVICE=${REGISTER_SERVICE}
    export CTX_XDL_ADD_FIREWALL_RULES=${ADD_FIREWALL_RULES}
    export CTX_XDL_AD_INTEGRATION=${CTX_AD_INTEGRATION}
    export CTX_XDL_HDX_3D_PRO=${HDX_3D_PRO}
    export CTX_XDL_VDI_MODE=${VDI_MODE}
    export CTX_XDL_SITE_NAME=${SITE_NAME}
    export CTX_XDL_LDAP_LIST=${LDAP_LIST}
    export CTX_XDL_SEARCH_BASE=${SEARCH_BASE}
    export CTX_XDL_FAS_LIST=${FAS_LIST}
    export CTX_XDL_START_SERVICE=${START_SERVICE}

    log_echo "CTX_XDL_SUPPORT_DDC_AS_CNAME=${CTX_XDL_SUPPORT_DDC_AS_CNAME}"
    log_echo "CTX_XDL_DDC_LIST=${CTX_XDL_DDC_LIST}"
    log_echo "CTX_XDL_VDA_PORT=${CTX_XDL_VDA_PORT}"
    log_echo "CTX_XDL_REGISTER_SERVICE=${CTX_XDL_REGISTER_SERVICE}"
    log_echo "CTX_XDL_ADD_FIREWALL_RULES=${CTX_XDL_ADD_FIREWALL_RULES}"
    log_echo "CTX_XDL_AD_INTEGRATION=${CTX_XDL_AD_INTEGRATION}"
    log_echo "CTX_XDL_HDX_3D_PRO=${CTX_XDL_HDX_3D_PRO}"
    log_echo "CTX_XDL_VDI_MODE=${CTX_XDL_VDI_MODE}"
    log_echo "CTX_XDL_SITE_NAME=${CTX_XDL_SITE_NAME}"
    log_echo "CTX_XDL_LDAP_LIST=${CTX_XDL_LDAP_LIST}"
    log_echo "CTX_XDL_SEARCH_BASE=${CTX_XDL_SEARCH_BASE}"
    log_echo "CTX_XDL_FAS_LIST=${CTX_XDL_FAS_LIST}"
    log_echo "CTX_XDL_START_SERVICE=${CTX_XDL_START_SERVICE}"

    for ((retry_num=1; retry_num<=$retry_max; retry_num++))
    do
        log_echo "Retry time: $retry_num, sleeping 2 seconds before running ctxsetup.sh..."
        sleep 2
        /opt/Citrix/VDA/sbin/ctxsetup.sh  2>&1 >> "$logFile"
        if [[ "$?" -ne "0" ]]; then
            log_echo "Error: Run ctxsetup.sh failed!"
        else
            log_echo "Run ctxsetup.sh succeed!"
            break
        fi
    done

    # configure local settings
    configure_local_settings

    log "Debug: Exit setup_vda"
}


function configure_local_settings()
{
    log "Debug: Enter configure_local_settings"
    /opt/Citrix/VDA/bin/ctxreg load -i /var/xdl/mcs/mcs_local_setting.reg || return 1
    log "Debug: Exit configure_local_settings"
}
